#include "cvCardAnalysis.h"


//-----------------------------
void cvCardAnalysis::setup(int w, int h){

	cardWarpW = 64;
	cardWarpH = 80;

	cardImg.allocate(cardWarpW, cardWarpH);
	thresholdedFrame.allocate(cardWarpW, cardWarpH);
	
	threshPixels = new unsigned char[cardWarpW * cardWarpH * 1];
}

//------------------------------------------
void cvCardAnalysis::getCardAsCvImage(card &theCard, ofxCvGrayscaleImage &videoFrame){
		
		int sx = theCard.boundingRect.x;
		int sy = theCard.boundingRect.y;
		int sw = theCard.boundingRect.width;
		int sh = theCard.boundingRect.height;
			
		ofxPoint2f tmpQuad[4];
		ofxPoint2f tmpDst[4];
				
		tmpDst[0].x = 0;
		tmpDst[0].y = 0;
		tmpDst[1].x = cardWarpW;
		tmpDst[1].y = 0;
		tmpDst[2].x = cardWarpW;
		tmpDst[2].y = cardWarpH;
		tmpDst[3].x = 0;
		tmpDst[3].y = cardWarpH;
		
		int pta = theCard.pta;
		int ptb = theCard.ptb;
		int ptc = theCard.ptc;
		int ptd = theCard.ptd;

		tmpQuad[0].x = theCard.c[pta].x;
		tmpQuad[0].y = theCard.c[pta].y;
		tmpQuad[1].x = theCard.c[ptb].x;
		tmpQuad[1].y = theCard.c[ptb].y; 										
		tmpQuad[2].x = theCard.c[ptc].x;
		tmpQuad[2].y = theCard.c[ptc].y;
		tmpQuad[3].x = theCard.c[ptd].x;
		tmpQuad[3].y = theCard.c[ptd].y; 
		
		cardImg.warpIntoMe(videoFrame, tmpQuad, tmpDst);
						
}

//------------------------------------------
void cvCardAnalysis::dynamicThreshold(int threshold){
	
	unsigned char * pixels = cardImg.getPixels();
	int totalPixels = cardWarpW * cardWarpH;
	
	int darkest = 255;
	int brightest = 0;

	//first find the brightest and darkest pixels
	unsigned char pix;		
	for(int i = 0; i < totalPixels; i++){
		pix = pixels[i];
		if(pix > brightest) brightest = pix;
		if(pix < darkest) darkest = pix;
	}	
	
	//then do the threshold
	float pct = 0.0;
	float range = (float)(brightest - darkest);
	
	if(range > 0){
		for(int i = 0; i < totalPixels; i++){
			pct = 255.0  * ( (float)(pixels[i] - darkest)/range );
			
			if(pct > threshold) threshPixels[i] = 0;
			else threshPixels[i] = 255;									
		}
	}else{
		for(int i = 0; i < totalPixels; i++)threshPixels[i] = 0;
	}
	 
	
	/*thresholdedFrame = cardImg; //
	thresholdedFrame.contrastStretch();
	thresholdedFrame.threshold(127);
	*/
	thresholdedFrame.setFromPixels(threshPixels, cardWarpW, cardWarpH);
	
	//thresholdedFrame.invert();
}

//------------------------------------------
void cvCardAnalysis::calculateHuMoments(card &theCard, bool bUseThresholdedFrame){

	if( bUseThresholdedFrame ){
		cvMoments(thresholdedFrame.getCvImage(), &theCard.Moments);
	}else{
		cvMoments(cardImg.getCvImage(), &theCard.Moments);	
	}
	cvGetHuMoments(&theCard.Moments, &theCard.HuMoments);
	
}









