#include "cvManager.h"





cvManager::cvManager(){
	bSetup				= false;
	which				= 0;	
	cornerSensitivity	= 5;
	activityThresh		= 3.8;
}

//changing cameras or switching from/to the camera mode
//requires the app to be restarted - mabe we can change this?
//---------------------------		
void cvManager::setupCamera(int deviceNumber, int width, int height){
	VG.setVerbose(true);
	//VG.setDeviceID(deviceNumber);
	VG.initGrabber(width, height);
	W = VG.width;
	H = VG.height;			
	bCameraSetup = true;
	which = 2;

	setupCV();
}
		
//same as above - from/to using test movies
//requires a restart
//---------------------------		
void cvManager::setupVideo(string videoPath){
	
	VP.loadMovie(videoPath);
	VP.play(); 
	VP.setPosition(0.0);
	VP.setSpeed(0.90);
	VP.setUseTexture(true);
	W = VP.width;
	H = VP.height;
	bVideoSetup = true;
	which = 1;
	
	setupCV();
}

//good for adjusting the color balance, brightness etc
//---------------------------		
void cvManager::openCameraSettings(){
	if(bCameraSetup)VG.videoSettings();			
}

//---------------------------		
void cvManager::useCamera(){
	if(bCameraSetup)which = 2;	
}		

//---------------------------		
void cvManager::useVideo(){
	if(bVideoSetup)which = 1;	
}	


//do all our openCV allocation
//---------------------------		
void cvManager::setupCV(){
	
	if(bSetup) return;
	
	cardAnal.setup(CARD_CV_W, CARD_CV_H);
			
	//our openCV inits
	VideoFrame.allocate(W, H);
	PresenceFrame.allocate(W, H);
	GreyFrame.allocate(W, H);
	blobTrack.setup(W, H, 200);
	
	bSetup = true;
}

		
//the heart of the beast - where we process 
//---------------------------		
int cvManager::findCards(card * cards, int threshold, int minSize, int maxSize){
			
	///////////////////////////////////////////////////////////
	// Part 1 - get the video data
	///////////////////////////////////////////////////////////
	
	//pointer to our incoming video pixels			
	unsigned char * pixCam;
	
	//either grab pixels from video or grab from camera
	if(which == 1){
		VP.idleMovie();	
		pixCam 	= VP.getPixels();
	}
	else 
	if(which == 2){
		VG.grabFrame();
		pixCam 	= VG.getPixels();		
	}else{
		return 0; // no blobs??				
	}

	///////////////////////////////////////////////////////////
	// Part 2 - load into openCV and make greyscale
	///////////////////////////////////////////////////////////
	VideoFrame.setFromPixels(pixCam, W, H);
	GreyFrame.setFromColorImage(VideoFrame);
	
	///////////////////////////////////////////////////////////
	// Part 3 - do thresholding
	///////////////////////////////////////////////////////////
	PresenceFrame = GreyFrame;	
	PresenceFrame.threshold(threshold);
	//PresenceFrame.erode_3x3();

	///////////////////////////////////////////////////////////
	// Part 4 - find the largest blobs of possible candidates 
	////////////////////////////////////////////////////////////
	int numBlobs = Contour.findContours(PresenceFrame, minSize, maxSize, 50, false);  
	int numBlobsZach = ContourForZach.findContours(PresenceFrame, minSize, maxSize, 50, false, false);  
	
	
	if( numBlobs == 0) return 0;
	
	///////////////////////////////////////////////////////////
	// Part 5 - do shit with the data
	////////////////////////////////////////////////////////////
	blobTrack.identifyBlobs(Contour.blobs, numBlobs, 20);

				
	///////////////////////////////////////////////////////////
	// Part 6 - For each card we find the four corners and calcuate the hu moments for each card
	////////////////////////////////////////////////////////////	
				
	int k = 0;
	for(int i = 0; i < numBlobs; i++){

		if( k >= NUM_CARDS ) break;
		
		//some things that will cause us to skip a card
		if(blobTrack.getBlobActivity(i) > activityThresh) continue;	 
		if(Contour.blobs[i].area < MIN_CARD_AREA) continue;
		
		//smooth the contour
		ofxVec2f * smoothContour = contAna.smoothContour(Contour.blobs[i], 0.1);
		
		//get the number of corners
		int numFound = corner.findCorners(smoothContour, Contour.blobs[i].nPts, cornerSensitivity);
		
		
		for (int j = 0; j <  ContourForZach.blobs[i].nPts; j++){
			
			// can't remember what this was for: 
			cards[k].CFB.lengthSection = 1; //((float)blobLen / 600.0f);		// useful to make the angle stuff make sense across diff. blob sizes.
			ofxPoint2f pt;
			pt.x = ContourForZach.blobs[i].pts[j].x;
			pt.y = ContourForZach.blobs[i].pts[j].y;
			
			if (j == 0){
				cards[k].CFB.start(pt);
			} else {
				cards[k].CFB.addPoint(pt);
			}
			
			//cards[k].CFB.heavilySmoothContour();
			cards[k].CFB.processContour();
		}
		 
		
		if(numFound != 4){
			//try and find corners with zach's line detector
			continue; //replace with line detector
		}
		
		
		
		
		cards[k].setPoints(smoothContour[corner.getTipIndex(0)],  smoothContour[corner.getTipIndex(1)], smoothContour[corner.getTipIndex(2)], smoothContour[corner.getTipIndex(3)]);
		cardGeom.calculateGeometry(cards[k]);
		
		cardAnal.getCardAsCvImage(cards[k],  GreyFrame);
		cardAnal.dynamicThreshold();
		cardAnal.calculateHuMoments(cards[k], true);
				
		k++;
	}
	
	///////////////////////////////////////////////////////////
	// Part 8 - we now return how many cards we found
	////////////////////////////////////////////////////////////	
	
	int numLegitCards = k;
	return numLegitCards;			
} 



// [fix] this is repetitive, no?

//the heart of the beast - where we process 
//---------------------------		
int cvManager::findCardFromImage	(card & myCard, int threshold, int minSize, int maxSize, ofImage & img){
	
	///////////////////////////////////////////////////////////
	// Part 1 - get the video data
	///////////////////////////////////////////////////////////
	
	//pointer to our incoming video pixels			
	unsigned char * pix = img.getPixels();
	
	///////////////////////////////////////////////////////////
	// Part 2 - load into openCV and make greyscale
	///////////////////////////////////////////////////////////
	VideoFrame.setFromPixels(pix, W, H);
	GreyFrame.setFromColorImage(VideoFrame);
	
	///////////////////////////////////////////////////////////
	// Part 3 - do thresholding
	///////////////////////////////////////////////////////////
	PresenceFrame = GreyFrame;	
	PresenceFrame.threshold(threshold);
	//PresenceFrame.erode_3x3();
	
	///////////////////////////////////////////////////////////
	// Part 4 - find the largest blobs of possible candidates 
	////////////////////////////////////////////////////////////
	int numBlobs = Contour.findContours(PresenceFrame, minSize, maxSize, 50, false);  
	if( numBlobs == 0) return 0;
	
	///////////////////////////////////////////////////////////
	// Part 5 - do shit with the data
	////////////////////////////////////////////////////////////
	blobTrack.identifyBlobs(Contour.blobs, numBlobs, 20);
	
	
	///////////////////////////////////////////////////////////
	// Part 6 - For each card we find the four corners and calcuate the hu moments for each card
	////////////////////////////////////////////////////////////	
	
	int k = 0;
	for(int i = 0; i < numBlobs; i++){
		
		if( k >= 1 ) break;
		
		//some things that will cause us to skip a card
		//if(blobTrack.getBlobActivity(i) > activityThresh) continue;	 
		if(Contour.blobs[i].area < MIN_CARD_AREA) continue;
		
		//smooth the contour
		ofxVec2f * smoothContour = contAna.smoothContour(Contour.blobs[i], 0.2);  //[zach] had to up this to get this to work
		
		//get the number of corners
		int numFound = corner.findCorners(smoothContour, Contour.blobs[i].nPts, cornerSensitivity);
		

		if(numFound != 4){			
			//try and find corners with zach's line detector
			continue; //replace with line detector
		}
		
		
		
		
		
		
		myCard.setPoints(smoothContour[corner.getTipIndex(0)],  smoothContour[corner.getTipIndex(1)], smoothContour[corner.getTipIndex(2)], smoothContour[corner.getTipIndex(3)]);
		cardGeom.calculateGeometry(myCard);
		
		cardAnal.getCardAsCvImage(myCard,  GreyFrame);
		
		myCard.myCardImage.setFromPixels(cardAnal.cardImg.getPixels(), CARD_CV_W, CARD_CV_H, OF_IMAGE_GRAYSCALE);
		
		// no thresh...
		cardAnal.dynamicThreshold();
		cardAnal.calculateHuMoments(myCard, true);
		
		k++;
	}
	
	///////////////////////////////////////////////////////////
	// Part 8 - we now return how many cards we found
	////////////////////////////////////////////////////////////	
	
	int numLegitCards = k;
	return numLegitCards;			
} 


//--------------------------
void cvManager::resetBlobID(){
	blobTrack.resetCounter();
}
		
//--------------------------
void cvManager::toggleVideoPlayback(){
	if(VP.getSpeed() == 0.0){
		VP.setSpeed(1.0);
	}else{
		VP.setSpeed(0.0);
	}
}

		
//---------------------------				
void cvManager::draw(float x, float y){
	ofSetColor(255, 255, 255);
	PresenceFrame.draw(x, y);
}



